'use strict';
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
const assert = require('assert');
const { ObjectId } = require('mongoose').Types;

//
class GoodsTagsService extends CrudService {
  constructor(ctx) {
    super(ctx, 'goodstags');
    this.model = this.ctx.model.System.GoodsTags;
  }
  async beforeQuery(filter) {
    const pid = _.get(filter, 'pid');
    if (!pid) filter.pid = { $exists: false };
    return filter;
  }

  /**
   * 树形图结构
   * @param {Object} query 查询条件
   * @param query.pid 一级id
   * @return {Array} 树形图结构数据
   */
  async tree({ pid }) {
    let list = await this.model.find({ status: '0' });
    list = JSON.parse(JSON.stringify(list));
    list = list.map((i) => _.omit(i, ['meta', '__v']));
    const level1 = list.filter((f) => (pid ? f.id === pid : !f.pid));
    const loop = (list, parents) => {
      for (const p of parents) {
        const pid = _.get(p, '_id');
        if (!pid) continue;
        const children = list.filter((f) => ObjectId(pid).equals(f.pid));
        if (children.length <= 0) continue;
        const newChildren = loop(list, children);
        p.children = newChildren;
      }
      return parents;
    };
    const tree = loop(list, level1);
    return tree;
  }

  async getData({ code }) {
    const data = await this.model.findOne({ code });
    return data;
  }
}

module.exports = GoodsTagsService;
